// Do main tunnel management (creaton, data manipulation, 
//		drawing)

#include "..\Headers\core.h"
#include "..\Headers\tunnel.h"

#include <math.h>

Vertex Vertexes[NumberOfCircles][PointsPerCircle+1];	// +1 because we need to close ower conic (tube, tunnel)
Vertex DefVertexes[NumberOfCircles][PointsPerCircle+1];	// the deformated Vertexes

void InitConic() {
	GLfloat length = (2 * myPI * CircleRadius / PointsPerCircle);
	GLfloat finalKTimesSmaller = 20;

	for(int i = 0 ; i < NumberOfCircles ; i++) {	// foreach(circle in totalCircles) 
		GLfloat scale = 1.0f + (float)(( ((float)i) * finalKTimesSmaller) / ((float)NumberOfCircles));
		GLfloat depth = -(i*DistanceBetweenCircles);
		
		Vertexes[i][0].x = CircleRadius / scale;
		Vertexes[i][0].y = 0 / scale;
		Vertexes[i][0].z = depth / scale;

		for(int j = 1 ; j < PointsPerCircle ; j++)
		{
			GLfloat pointLength = j * length;
			GLfloat alpha = pointLength / CircleRadius;

			Vertexes[i][j].x = CircleRadius*cos(alpha) / scale;
			Vertexes[i][j].y = CircleRadius*sin(alpha) / scale;
			Vertexes[i][j].z = depth / scale;
		}

		Vertexes[i][j] = Vertexes[i][0];
	}
}

float deformationQuanta		= 3*myPI/2;	// deforming with a PI/2 value from the sin
float deformingStartRegion	= 0;		// the start region from whitch to deform
float deformingScaleX		= 0.6f;
float sgnx = -1, sgny = -1;
float deformingScaleY		= 0.6f;
float deformingIncrement	= 1.5*(myPI/180);	// increment , in degrees
void ApplyEcuationToConic() {
	// Current used ecuation is the sin
	
	float mySin = sin(myPI / 5);

	for(int i = 0 ; i < NumberOfCircles ; i++) {	// foreach(circle in totalCircles) 
		float alpha = (deformationQuanta*((float)i))/NumberOfCircles;
		float defX = (deformingScaleX*sin(alpha+deformingStartRegion));
		float defY = (deformingScaleY*cos(alpha+deformingStartRegion));
		
		for(int j = 0 ; j < PointsPerCircle ; j++)
		{
			DefVertexes[i][j].x = Vertexes[i][j].x + defX;
			DefVertexes[i][j].y = Vertexes[i][j].y + defY;
			DefVertexes[i][j].z = Vertexes[i][j].z;
		}

		DefVertexes[i][j] = DefVertexes[i][0];
	}

	deformingScaleX += sgnx * float(rand() % 10) / 2000;
	if(deformingScaleX >= 1 || deformingScaleX <= .5)
		sgnx *= -1;

	deformingScaleY += sgny * float(rand() % 10) / 2000;
	if(deformingScaleY >= 1 || deformingScaleY <= .5)
		sgny *= -1;

	deformingStartRegion += deformingIncrement;
	if(deformingStartRegion >= myPI*2)
		deformingStartRegion -= myPI*2;
}

float displacementX = 0, displacementY = 0;
float displacement_incrementX = 0.1, displacement_incrementY = 0.03f;
int DrawConic() {

	if(renderConicWireframe)
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

	gluLookAt(0.0f, 0.0f, 1.0f,	
				//0.0f, 0.0f, 0.0f,	
				(DefVertexes[0][0].x-Vertexes[0][0].x)/4, 
				(DefVertexes[0][0].y-Vertexes[0][0].y)/4, 
				(DefVertexes[0][0].z-Vertexes[0][0].z)/4,
				0.0f, 1.0f, 0.0f);

	glBindTexture(GL_TEXTURE_2D, conic_texture);

	glBegin(GL_QUADS);//GL_TRIANGLES);
		for(int i = 0 ; i < (NumberOfCircles-1); i++) {
			float alpha1 = ((float)i) / ((float)NumberOfCircles);
			float alpha2 = ((float)i+1) / ((float)NumberOfCircles);
			for(int j = 0 ; j < PointsPerCircle ; j++) {
				
				if(useNormals) {
					float x1, x2, y1, y2, z1, z2, vNormalx, vNormaly, vNormalz;
							x1 = DefVertexes[i][j].x - DefVertexes[i+1][j].x;
							x2 = DefVertexes[i+1][j].x - DefVertexes[i][j+1].x;

							y1 = DefVertexes[i][j].y - DefVertexes[i+1][j].y;
							y2 = DefVertexes[i+1][j].y - DefVertexes[i][j+1].y;

							z1 = DefVertexes[i][j].z - DefVertexes[i+1][j].z;
							z2 = DefVertexes[i+1][j].z - DefVertexes[i][j+1].z;

							vNormalx = ((y1 * z2) - (z1 * y2));
							vNormaly = ((z1 * x2) - (x1 * z2));
							vNormalz = ((x1 * y2) - (y1 * x2));

							float mag = (float)sqrt( (vNormalx * vNormalx) + (vNormaly * vNormaly) + (vNormalz * vNormalz) );

							vNormalx /= mag;
							vNormaly /= mag;
							vNormalz /= mag;	

					glNormal3f(vNormalx, vNormaly, vNormalz);
				}	

				glColor3f(1.0f - alpha1, 1.0f - alpha1, 1.0f - alpha1);
				glTexCoord2f(0.0f + displacementX, 0.0f + displacementY);
				glVertex3f(DefVertexes[i][j].x, DefVertexes[i][j].y, DefVertexes[i][j].z);

				glColor3f(1.0f - alpha1, 1.0f - alpha1, 1.0f - alpha1);
				glTexCoord2f(0.0f + displacementX, 1.0f + displacementY);
				glVertex3f(DefVertexes[i][j+1].x, DefVertexes[i][j+1].y, DefVertexes[i][j+1].z);

				glColor3f(1.0f - alpha2, 1.0f - alpha2, 1.0f - alpha2);
				glTexCoord2f(1.0f + displacementX, 1.0f + displacementY);
				glVertex3f(DefVertexes[i+1][j+1].x, DefVertexes[i+1][j+1].y, DefVertexes[i+1][j+1].z);

				glColor3f(1.0f - alpha2, 1.0f - alpha2, 1.0f - alpha2);
				glTexCoord2f(1.0f + displacementX, 0.0f + displacementY);
				glVertex3f(DefVertexes[i+1][j].x, DefVertexes[i+1][j].y, DefVertexes[i+1][j].z);
			}
		}
	glEnd();

	displacementX += displacement_incrementX;
	displacementY += displacement_incrementY;

	if(renderConicWireframe) 
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	return TRUE;
}